unit FormMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Objects, IPPeerClient, IPPeerServer, System.Tether.Manager,
  System.Tether.AppProfile, FMX.Ani, FMX.Layouts, FMX.ListBox;

type
  TMainForm = class(TForm)
    Header: TToolBar;
    Footer: TToolBar;
    HeaderLabel: TLabel;
    dialVolumen: TArcDial;
    tbCancion: TTrackBar;
    lblTiempoCancion: TLabel;
    Label1: TLabel;
    Switch: TSwitch;
    sbPlay: TSpeedButton;
    sbPause: TSpeedButton;
    sbAdelante: TSpeedButton;
    sbStop: TSpeedButton;
    sbAtras: TSpeedButton;
    imgCaratula: TImage;
    lblAlbum: TLabel;
    lblCancion: TLabel;
    lblTiempoTotal: TLabel;
    pbVolumen: TProgressBar;
    lblVolMIn: TLabel;
    lblVolMax: TLabel;
    lblArtista: TLabel;
    lblGenero: TLabel;
    lblAnyo: TLabel;
    rectGridSup: TRectangle;
    ttManager: TTetheringManager;
    ttProfiler: TTetheringAppProfile;
    GradientAnimation1: TGradientAnimation;
    sbLista: TSpeedButton;
    pnlListaCanciones: TPanel;
    lbCanciones: TListBox;
    Label2: TLabel;
    imgWifi: TImage;
    imgBluetooth: TImage;
    procedure dialVolumenChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SwitchSwitch(Sender: TObject);
    procedure ttManagerEndManagersDiscovery(const Sender: TObject;
      const RemoteManagers: TTetheringManagerInfoList);
    procedure ttProfilerResourceUpdated(const Sender: TObject;
      const AResource: TRemoteResource);
    procedure ttManagerEndProfilesDiscovery(const Sender: TObject;
      const RemoteProfiles: TTetheringProfileInfoList);
    procedure ttProfilerAcceptResource(const Sender: TObject;
      const AProfileId: string; const AResource: TCustomRemoteItem;
      var AcceptResource: Boolean);
    procedure ttProfilerResourceReceived(const Sender: TObject;
      const AResource: TRemoteResource);
    procedure sbPlayClick(Sender: TObject);
    procedure ttManagerNewManager(const Sender: TObject;
      const AManagerInfo: TTetheringManagerInfo);
    procedure ttManagerPairedFromLocal(const Sender: TObject;
      const AManagerInfo: TTetheringManagerInfo);
    procedure ttManagerPairedToRemote(const Sender: TObject;
      const AManagerInfo: TTetheringManagerInfo);
    procedure imgCaratulaClick(Sender: TObject);
    procedure sbListaClick(Sender: TObject);
    procedure lbCancionesClick(Sender: TObject);
    procedure tbCancionChange(Sender: TObject);
    procedure imgWifiClick(Sender: TObject);
    procedure imgBluetoothClick(Sender: TObject);
    procedure ttManagerUnPairManager(const Sender: TObject;
      const AManagerInfo: TTetheringManagerInfo);
  private
    changing:boolean;
    imgHeight, imgTop, imgLeft:Single;

    // Devuelve si hay alguien conectado
    function AppConectada():boolean;
    // Limpiar los controles de pantalla
    procedure ClearInfo();

    procedure _Log(AMsg:string);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.fmx}
{$R *.NmXhdpiPh.fmx ANDROID}
{$R *.SmXhdpiPh.fmx ANDROID}

uses
  Math;

//
  function DialToVolume(DialVolume:double):integer;
  var
    iVol:integer;
    fVol:double;
  begin
    fVol := DialVolume;
    //
    if (fVol <= -40) and (fVol >= -180) and (fVol < 0) then begin
      iVol := Round((Abs(fVol) - 40) * 0.35);
    end;
    if (fVol >= 40) and (fVol <= 180) and (fVol > 0) then begin
      iVol := Round((Abs(fVol) - 40) * 0.35);
      iVol := (100 - iVol);
    end;
    Result := iVol;
  end;
  //
  function VolumeToVolume(AVolume:integer):double;
  var
    iVol:integer;
    fVol:double;
  begin
    // fVol := DialVolume;
    iVol := AVolume;

    if (iVol <= 50) then begin
      fVol := -(iVol / 0.35) - 40;
    end;

    if (iVol >= 50) then begin
      iVol := Trunc((iVol / 0.35)) + 40;
    end;

    Result := fVol;

//    //
//    if (fVol <= -40) and (fVol >= -180) and (fVol < 0) then begin
//      iVol := Round((Abs(fVol) - 40) * 0.35);
//    end;
//    if (fVol >= 40) and (fVol <= 180) and (fVol > 0) then begin
//      iVol := Round((Abs(fVol) - 40) * 0.35);
//      iVol := (100 - iVol);
//    end;
//    Result := iVol;
  end;
//




function TMainForm.AppConectada: boolean;
begin
  Result := (ttProfiler.ConnectedProfiles.Count > 0){ and
            (ttManager.RemoteManagers.Count > 0)};
end;

// Limpiar los controles de pantalla
procedure TMainForm.ClearInfo();
begin

  // Limpiar los componentes
  lblArtista.Text := 'Artista: ';
  lblAlbum.Text := 'Album: ';
  lblCancion.Text := '';
  lblGenero.Text := 'Gnero: ';
  lblAnyo.Text := 'Ao: ';
  pbVolumen.Value := 0;
  dialVolumen.Value := 0;
  lblTiempoTotal.Text := '...';
  lblTiempoCancion.Text := '...';
  tbCancion.Value := 0;
  tbCancion.Max := 100;
  lbCanciones.Items.Clear;
  // Limpiar la imagfen
  imgCaratula.Bitmap.Assign(nil);


end;

procedure TMainForm.dialVolumenChange(Sender: TObject);
begin

  // Si no estamos conectados, salimos
  if not AppConectada then Exit;

  if Changing then Exit;

  if (dialVolumen.Value > -40) and (dialVolumen.Value < 0)  then begin
    changing := true;
    try
      dialVolumen.Value := -40;
    finally
      changing := False;
    end;
  end;

  if (dialVolumen.Value < 40) and (dialVolumen.Value > 0)  then begin
    changing := true;
    try
      dialVolumen.Value := 40;
    finally
      changing := False;
    end;
  end;

  // Volumen
  pbVolumen.Value := DialToVolume(Trunc(dialVolumen.Value));

  // Para enviar
  ttProfiler.Resources.FindByName('MasterVolume').Value := IntToStr(Trunc(pbVolumen.Value));

end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  // ini
  dialVolumen.Value := -40;
  // restangulos
  rectGridSup.Visible := True;
  rectGridSup.Align := TAlignLayout.Client;
  Footer.Visible := Switch.IsChecked;

  // Lista canciones
  pnlListaCanciones.Visible := False;
end;

procedure TMainForm.imgBluetoothClick(Sender: TObject);
begin
  ttManager.Enabled := False;
  ttManager.AllowedAdapters := 'Network';
  imgWifi.Visible := True;
  imgbluetooth.Visible := False;
  imgbluetooth.Align := TAlignLayout.FitLeft;
  ttManager.Enabled := True;
end;

procedure TMainForm.imgCaratulaClick(Sender: TObject);
begin
  if (imgCaratula.Height < 60) then begin
    imgHeight := imgCaratula.Height;
    imgLeft := imgCaratula.Position.X;
    imgTop := imgCaratula.Position.Y;
    imgCaratula.Height := imgHeight * 3;
    imgCaratula.Width := imgHeight * 3;
    imgCaratula.Align := TAlignLayout.Center;
    imgCaratula.WrapMode := TImageWrapMode.Stretch;

    rectGridSup.Visible := True;
    rectGridSup.Align := TAlignLayout.Client;
    rectGridSup.BringToFront;

    imgCaratula.BringToFront;
  end
  else begin
    rectGridSup.Visible := False;
    imgCaratula.Align := TAlignLayout.None;
    imgCaratula.Height := imgHeight;
    imgCaratula.Width := imgHeight;
    imgCaratula.Position.X := imgLeft;
    imgCaratula.Position.Y := imgTop;
    imgCaratula.WrapMode := TImageWrapMode.Fit;
  end;

end;

procedure TMainForm.imgWifiClick(Sender: TObject);
begin

  ttManager.Enabled := False;
  ttManager.AllowedAdapters := 'Bluetooth';
  imgWifi.Visible := False;
  imgbluetooth.Visible := True;
  imgbluetooth.Align := TAlignLayout.FitLeft;
  ttManager.Enabled := True;

end;

procedure TMainForm.lbCancionesClick(Sender: TObject);
begin
  pnlListaCanciones.Visible := False;
end;

procedure TMainForm.sbListaClick(Sender: TObject);
begin
  // Lista
  if (pnlListaCanciones.Visible = True) then begin
    pnlListaCanciones.Visible := False;
  end
  else begin
    pnlListaCanciones.Visible := True;
    pnlListaCanciones.Align := TAlignLayout.Client;
    pnlListaCanciones.BringToFront;
  end;

end;

procedure TMainForm.sbPlayClick(Sender: TObject);
var
  ra:TRemoteAction;
  pInfo:TTetheringProfileInfo;
begin
  // Hemos encontrado alguno?
  if (ttManager.RemoteProfiles.Count > 0) then begin
    pInfo := ttManager.RemoteProfiles[0];

    // Ejecutamos acciones remotas
    if(TComponent(Sender).Tag = 0) then begin
      ttProfiler.RunRemoteAction(pInfo, 'rPlay');
    end
    else if(TComponent(Sender).Tag = 1) then begin
      ttProfiler.RunRemoteAction(pInfo, 'rPause');
    end
    else if(TComponent(Sender).Tag = 2) then begin
      ttProfiler.RunRemoteAction(pInfo, 'rStop');
    end
    else if(TComponent(Sender).Tag = 3) then begin
      ttProfiler.RunRemoteAction(pInfo, 'rAdelante');
    end
    else if(TComponent(Sender).Tag = 4) then begin
      ttProfiler.RunRemoteAction(pInfo, 'rAtras');
    end;
  end;

end;

procedure TMainForm.SwitchSwitch(Sender: TObject);
begin

  if (Switch.IsChecked) then begin
    // Buscar otras aplicaciones
    ttManager.DiscoverManagers(3000);

    // Ocultar botones de cobnexion
    imgWifi.Visible := FalsE;
    imgBluetooth.Visible := FalsE;

    Exit;
  end
  else begin
    // Hay alguien conectado?
    if AppConectada then begin
      // desconectar
      ttManager.UnPairManager(ttManager.RemoteManagers[0]);
      //ttManager.DiscoverProfiles(mInfo);
      Footer.Visible := False;
      rectGridSup.Visible := True;
      // Limpiar controles
      ClearInfo;
    end;
  end;

end;

procedure TMainForm.tbCancionChange(Sender: TObject);
begin

  // Si no estamos conectados, salimos
  if not AppConectada then Exit;

  // Para enviar
  ttProfiler.Resources.FindByName('PosicionCancion').Value := IntToStr(Trunc(tbCancion.Value));
end;

procedure TMainForm.ttManagerEndManagersDiscovery(const Sender: TObject;
  const RemoteManagers: TTetheringManagerInfoList);
var
  mInfo:TTetheringManagerInfo;
begin
  // Hemos encontrado alguno?
  if (ttManager.RemoteManagers.Count > 0) then begin
    // Emparejar el seleccionado
    mInfo := ttManager.RemoteManagers[0];
    ttManager.PairManager(mInfo);
    //ttManager.DiscoverProfiles(mInfo);
    Footer.Visible := True;
    rectGridSup.Visible := False;
  end
  else begin
    ShowMessage('No se ha encontrado el reproductor.' + sLineBreak +
                'O no est en marcha o hay problemas de conexin.');
    Switch.IsChecked := False;
  end;

end;

procedure TMainForm.ttManagerEndProfilesDiscovery(const Sender: TObject;
  const RemoteProfiles: TTetheringProfileInfoList);
var
  i:integer;
  pInfo:TTetheringProfileInfo;
  rRes:TRemoteResource;
  b:boolean;
  ra:TRemoteAction;
begin

  if (not Switch.IsChecked) then Exit;

  // Suscribirse a los recursos
  i := ttManager.RemoteProfiles.Count;
  i := RemoteProfiles.Count;
  _Log('Encontrados profiles: ' + IntToStr(i));

  // Hay alguno?
  if (i > 0) then begin
    pInfo := ttManager.RemoteProfiles[0];
    pInfo := RemoteProfiles[0];

    b := ttProfiler.Connect(pInfo);
    if (b) then begin
      _Log('Connected');
    end;

    _Log('Profile[0]: ' + pInfo.ProfileIdentifier);
    _Log('Profile[0]: ' + pInfo.ProfileText);

    // Suscribirse a los recursos remotos
    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rArtista');
    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rAlbum');
    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rCancion');
    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rGenero');
    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rAnyo');
    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rVolumen');

    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rTiempoTotal');
    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rTiempoCancion');

    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rPosCancion');
    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'rMaxCancion');

    b := ttProfiler.SubscribeToRemoteItem(RemoteProfiles[0], 'ListaCanciones');

    // Pedir info de la cancin
    if(TComponent(Sender).Tag = 0) then begin
      //..MessageDlg('Pedir Info', TMsgDlgType.mtWarning, [TMsgDlgBtn.mbOK], 0);
      ttProfiler.RunRemoteAction(pInfo, 'ActionSendInfoToRemote');
    end
  end;

  // Visualizar
  rectGridSup.Visible := False{not Switch.IsChecked};
  Footer.Visible := true{Switch.IsChecked};

end;

procedure TMainForm.ttManagerNewManager(const Sender: TObject;
  const AManagerInfo: TTetheringManagerInfo);
begin
  _Log('On new manager');
end;

procedure TMainForm.ttManagerPairedFromLocal(const Sender: TObject;
  const AManagerInfo: TTetheringManagerInfo);
begin
  _Log('Paired from Local');
  // Ocultar botones de cobnexion
  imgWifi.Visible := FalsE;
  imgBluetooth.Visible := FalsE;
end;

procedure TMainForm.ttManagerPairedToRemote(const Sender: TObject;
  const AManagerInfo: TTetheringManagerInfo);
begin
  _Log('Paired To remote');
  // Ocultar botones de cobnexion
  imgWifi.Visible := FalsE;
  imgBluetooth.Visible := FalsE;
end;

procedure TMainForm.ttManagerUnPairManager(const Sender: TObject;
  const AManagerInfo: TTetheringManagerInfo);
begin
  // desconectar
  rectGridSup.Visible := True;
  Footer.Visible := False;
end;

procedure TMainForm.ttProfilerAcceptResource(const Sender: TObject;
  const AProfileId: string; const AResource: TCustomRemoteItem;
  var AcceptResource: Boolean);
begin
  AcceptResource := True;
end;

procedure TMainForm.ttProfilerResourceReceived(const Sender: TObject;
  const AResource: TRemoteResource);
begin

  _Log('Resource received; ' + AResource.Name);
  _Log('Hint= ' + AResource.Hint);
  _Log('Valor= ' + AResource.Value.AsString);

  // Revisar el recurso
  if (AResource.Hint = 'rAlbum') then begin
    lblAlbum.Text := AResource.Value.AsString;
  end
  else if (AResource.Hint = 'rCancion') then begin
    lblCancion.Text := AResource.Value.AsString;
  end
  else if (AResource.Hint = 'rArtista') then begin
    lblArtista.Text := AResource.Value.AsString;
  end
  else if (AResource.Hint = 'rGenero') then begin
    lblGenero.Text := AResource.Value.AsString;
  end
  else if (AResource.Hint = 'rCaratula') then begin
    imgCaratula.Bitmap.LoadFromStream(AResource.Value.AsStream);
  end;

end;

procedure TMainForm.ttProfilerResourceUpdated(const Sender: TObject;
  const AResource: TRemoteResource);
var
  Str:String;
begin

  Footer.Visible := Switch.IsChecked;
//  rectGridSup.Visible := not Switch.IsChecked;

  // recurso
  Str := AResource.Name;
  if (Str = 'rArtista') then begin
    lblArtista.Text := 'Artista: ' + AResource.Value.AsString;
  end
  else if (Str = 'rAlbum') then begin
    lblAlbum.Text := 'Album: ' + AResource.Value.AsString;
  end
  else if (Str = 'rCancion') then begin
    lblCancion.Text := AResource.Value.AsString;
  end
  else if (Str = 'rGenero') then begin
    lblGenero.Text := 'Gnero: ' + AResource.Value.AsString;
  end
  else if (Str = 'rAnyo') Then begin
    lblAnyo.Text := 'Ao: ' + AResource.Value.AsString;
  end
  else if (Str = 'rVolumen') then begin
    pbVolumen.Value := StrToIntDef(AResource.Value.AsString, 0);
    dialVolumen.Value := VolumeToVolume(Trunc(pbVolumen.Value));
  end
  else if (Str = 'rTiempoTotal') then begin
    lblTiempoTotal.Text := AResource.Value.AsString;
  end
  else if (Str = 'rTiempoCancion') then begin
    lblTiempoCancion.Text := AResource.Value.AsString;
  end
  else if (Str = 'rPosCancion') then begin
    tbCancion.Value := StrToIntDef(AResource.Value.AsString, 0);
  end
  else if (Str = 'rMaxCancion') then begin
    tbCancion.Max := StrToIntDef(AResource.Value.AsString, 0);
  end
  else if (Str = 'ListaCanciones') then begin
    lbCanciones.Items.Text := AResource.Value.AsString;
  end;

//  Str := AResource.Value.AsString;
//  lblAlbum.Text := AResource.Value.AsString;

end;


procedure TMainForm._Log(AMsg: string);
begin
  //...
//  ShowMessage('MANDO: ' + AMsg);
end;

end.
